use crate::pos::Pos;

#[derive(Clone, PartialEq, Debug)]
pub struct Stmt {
    pub kind: StmtKind,
    pub pos: Pos,
}

impl Stmt {
    pub fn new(kind: StmtKind, pos: Pos) -> Self {
        Self { kind, pos }
    }
}

#[derive(Clone, PartialEq, Debug)]
pub enum StmtKind {
    Error,
    Let {
        subject: Box<Param>,
        value: Box<Expr>,
    },
    Fn {
        subject: Box<Expr>,
        params: Vec<Param>,
        return_typ: Option<Box<Typ>>,
        body: Box<Expr>,
    },
    Return {
        expr: Option<Box<Expr>>,
    },
    Break {
        expr: Option<Box<Expr>>,
    },
    Assign {
        subject: Box<Expr>,
        value: Box<Expr>,
    },
    Expr(Box<Expr>),
}

#[derive(Clone, PartialEq, Debug)]
pub struct Expr {
    pub kind: ExprKind,
    pub pos: Pos,
}

impl Expr {
    pub fn new(kind: ExprKind, pos: Pos) -> Self {
        Self { kind, pos }
    }
}

#[derive(Clone, PartialEq, Debug)]
pub enum ExprKind {
    Error,
    Ident(u64),
    Int(i64),
    Str(String),
    Group(Box<Expr>),
    Block {
        stmts: Vec<Stmt>,
        expr: Option<Box<Expr>>,
    },
    Call {
        subject: Box<Expr>,
        args: Vec<Expr>,
    },
    If {
        cond: Box<Expr>,
        truthy: Box<Expr>,
        falsy: Option<Box<Expr>>,
    },
    Loop {
        body: Box<Expr>,
    },
}

#[derive(Clone, PartialEq, Debug)]
pub struct Typ {
    pub kind: TypKind,
    pub pos: Pos,
}

impl Typ {
    pub fn new(kind: TypKind, pos: Pos) -> Self {
        Self { kind, pos }
    }
}

#[derive(Clone, PartialEq, Debug)]
pub enum TypKind {
    Error,
    Ident(u64),
}

#[derive(Clone, PartialEq, Debug)]
pub struct Param {
    pub subject: Box<Expr>,
    pub typ: Option<Box<Typ>>,
    pub pos: Pos,
}
