#![allow(dead_code)]

use std::{rc::Rc, sync::Mutex};

use ast_lower::AstLower;
use parser::Parser;
use pos::{ErrorAcc, Pos};

mod ast;
mod ast_lower;
mod hir;
mod lexer;
mod parser;
mod pos;
mod token;

fn main() {
    let args = std::env::args().collect::<Vec<_>>();
    let filename = args[1].clone();
    let text = std::fs::read_to_string(filename).unwrap();

    let error_acc = Rc::new(Mutex::new(ErrorAcc::new()));

    let mut parser = Parser::new(&text, error_acc.clone());
    let ast = parser.parse();

    let mut ast_lower = AstLower::new(error_acc.clone());
    ast_lower.lower_ast(&ast);
    let pack = ast_lower.pack_it_up();

    println!("text: {text}");
    // println!("ast: {ast:#?}");
    // println!("hir: {pack:#?}");
    let symbols = parser.symbols();
    println!("symbols: {symbols:#?}");

    let error_acc = error_acc.lock().unwrap();
    if !error_acc.ok() {
        println!("Compilation failed");
        for error in error_acc.iter() {
            match error.pos {
                Some(Pos { line, col, .. }) => {
                    println!("{:?} at {line}:{col}: {}", error.kind, error.msg)
                }
                None => println!("{:?}: {}", error.kind, error.msg),
            }
        }
    }
}
